/*  RDADI.H

    Rendering ADI driver function requests. The rendering display
    device is either a high resolution color display device (e.g. PGC or
    Sun color monitor) or a comparable hardcopy device.

    modified 4/26/88 to change char * to char far * in init & fname
    
    modified 10/26/88 to do a variant between (char *) to (char far *)
		      in init & fname depending on a UNIX #define.
    modified 7/5/89 to be controlled by the RDADI_DEFINED compile
		    switch, as this include file is now needed by
		    UADI.H, and can still stand alone.

		      - WCA

*/


#ifndef RDADI_DEFINED
#define RDADI_DEFINED 1

#ifndef scrcoord
#define scrcoord short
#endif
#ifndef ushort
/* short rather than unsigned short for Lattice 2.15 */
#define ushort short
#endif

#ifndef real
#define real double
#endif

#ifdef UNIX
#define RDCFGBUFSIZE  128          /* Maximum Configuration data block size */
#define RDFNAMESIZE   128          /* Maximum filename string size. */
#endif

#define RDLINKUP 2000              /* Initialize rendering link up */
#define RDINIT   2001              /* Initialise use of rendering device */
#define RDSTART  2002              /* Start new rendering */
#define RDEND    2003              /* End rendering */
#define RDTERM   2004              /* Terminate use of rendering device */
#define RDRESIZE 2005              /* Change size of rendering screen */
#define RDCLEAR  2006              /* Clear entire display */
#define RDCMAPB  2007              /* Begin colour map */
#define RDCMAP   2008              /* Set colour map */
#define RDCMAPE  2009              /* End of colour map */
#define RDPOLY   2010              /* Polygon */
#define RDFNAME  2011              /* User supplied filename */
#define RDCRANGE 2012              /* Continuous colour range */
#define RDCPOLY  2013              /* Continuous colour polygon */

struct rd_rgb {                 /* Continuous colour RGB value item */
        real r, g, b;              /* Primary colour intensities */
};

struct rd_init {                /* Initialisation request (all reply cells) */
                                /* Also used for resize request */
        ushort flags;              /* Initialization flags */
        scrcoord xdots, ydots;     /* Total screen size in X and Y */
        ushort pixwid, pixhgt;     /* Pixel width and pixel height */
        short ncolour;             /* Number of rgb colour values */
        ushort maxintens;          /* Maximum intensity */
        ushort nshades;            /* Number of shades per colour */
#ifdef UNIX
        char *name;                /* Driver name string */
        char *rendconf;            /* Device configuration block */
#else
        char far *name;            /* Driver name string */
        char far *rendconf;        /* Device configuration block */
#endif
};

/* Values in flags of init packet */
#define RF_SINGLSCR   0x1          /* User requests single screen mode */
#define RF_REDRAWSCR  0x2          /* Redraw required on flip screen */
#define RF_FAILINIT   0x4          /* Driver could not initialize */
#define RF_FILEOUTPUT 0x8          /* Driver writes output to file */
#define RF_NAME       0x10         /* Driver is returning its name string */
/* The RF_HARDCOPY bit is both input and output.  On input, AutoSHADE
   indicates it does or does not want a hardcopy driver.  On output, driver
   will set it to the appropriate value (in the unlikely event it can do 
   either one, it can consider itself informed of which duty it is 
   supposed to perform). */
#define RF_HARDCOPY   0x20         /* This is Hard Copy device */
#define RF_DORESIZE   0x40         /* device to be polled for size */
/* If the following bit is set, all polygons will be passed to the
   rendering driver as scan lines rather than polygons. */
#define RF_SCANLINE   0x80         /* Device wants scan line output */
/* if the following bit is set, AutoShade will use polygon normals
   to compute smooth shading factors at each polygon vertex.  These
   shading factors will be passed to the device with each polygon. */
#define RF_SMOOTHFAC  0x100        /* Device wants smooth shade factors */

/* rd_start is passed to the driver everytime we start a new
   rendering. */

struct rd_start {                  /* Start rendering request */
        ushort flags;
};
#define RF_VIEW    0x1             /* viewing flag */

/* The structure RD_POLY is used to pass a polygon to a colour-mapped
   device. */

struct rd_poly {                   /* Polygon */
        ushort flags;              /* flags */
        short colour;              /* fill colour */
        short ecolour;             /* edge colour */
        short nvert;               /* number of verticies total */
        scrcoord vx[10];           /* next 10 verticies */
        scrcoord vy[10];
};

/* The structure RD_CPOLY is used to pass a polygon to a continuous
   colour device. */

struct rd_cpoly {                  /* Polygon in continuous colour */
        ushort flags;              /* flags */
        struct rd_rgb colour;      /* fill colour */
        struct rd_rgb ecolour;     /* edge colour */
        real shadef;               /* shading factor */
        short nvert;               /* number of verticies total */
        scrcoord vx[10];           /* next 10 verticies */
        scrcoord vy[10];
        real sf[10];               /* shade factor at each vertex */
};

/* Values in flags of rd_poly packet */
#define RF_MORE    0x1             /* this packet not complete */
#define RF_CONT    0x2             /* this packet is continuation of prev */
#define RF_LEFT    0x4             /* image intended for left eye only */
#define RF_RIGHT   0x8             /* image intended for right eye only */
#define RF_SF      0x10            /* shade factors specified */
/* top 10 bits reserved for invisible edges */

struct rd_cmap {                   /* Colour map */
        short code;                /* index */
        ushort red;                /* red intensity */
        ushort green;              /* green intensity */
        ushort blue;               /* blue intensity */
};

struct rd_crange {                 /* Colour range for continuous tones */
        real minshade;             /* Minimum shade multiplier */
        real maxshade;             /* Maximum shade multiplier */
        short rstretch;            /* Contrast stretch requested ? */
        struct rd_rgb bcolour;     /* Background colour request */
        real ambient;              /* Ambient contribution */
};

struct rd_fname {                  /* Filename */
        ushort flags;
#ifdef UNIX
        char *fname;               /* name entered by user */
        char *errormsg;            /* error message for AutoShade to put up */
        char *ftitle;              /* title for another file request */
#else
        char far *fname;           /* name entered by user */
        char far *errormsg;        /* error message for AutoShade to put up */
        char far *ftitle;          /* title for another file request */
#endif
};

/* Values in rd_fname flags */
#define RF_TITLE     0x1            /* Shade -> give me title */
#define RF_CANCEL    0x2            /* Shade -> cancelled by user */
#define RF_RECORDING 0x4            /* Shade -> recording file acquired */
#define RF_GETFNAME  0x8            /* Driver -> get another name */
#define RF_ERROR     0x10           /* Driver -> Error message supplied */

/*  Master rendering display driver request packet union  */

union rd_mstr {
        struct rd_init  rinit;     /* Initialise */
        struct rd_start rstart;    /* Start rendering */
        struct rd_poly  rpoly;     /* polygon verticies */
        struct rd_cmap  rcmap;     /* colour map entry */
        struct rd_fname rfname;    /* filename */
        struct rd_crange rcrange;  /* shading range */
        struct rd_cpoly rcpoly;    /* continuous tone polygon */
};

/*  Universal rendering display request / reply packet structure  */

struct rendreq {
        short rfunc;               /* Function / status */
        union rd_mstr rpkt;
};

/* Render slide header structure */

#define PRSLEN 40

struct rndhdr {
        char prs[PRSLEN];          /* primate readable string */
        char type;                 /* machine type */
        char level;                /* render slide level */
        scrcoord xdots, ydots;     /* total screen x and y size */
        short ncolour;             /* number of rgb colour values */
        ushort maxintens;          /* maximum intensity */
        ushort nshades;            /* number of shades per colour */
        ushort pixwid, pixhgt;     /* pixel width and height */
        short btest;               /* test for byte reversal */
};

#endif /* RDADI_DEFINED */
