/*	    XSIO.C			

    AutoCAD port I/O functions for the IBM AT PC running
    the XENIX operating system.

*/
/*	This program is copyrighted by Autodesk, Inc. and is
	furnished pursuant to a software license agreement.  The
	source code of this program may not be distributed or
	published in any form by the licensee.	Licensee is free to
	incorporate this code in object form in derivative works
	providing such derivative works are solely intended to work
	with Autodesk, Inc. products.  Said derivative works,
	incorporating object versions of this program may be
	distributed without royalty at the discretion of the licensee.

	Autodesk makes no warranties, express or implied, as to the
	correctness of this code or any derivative works which
        incorporate it.  Autodesk provides this code on an ``as-is''
	basis and explicitly disclaims any liability, express or
	implied, for errors, omissions, and other problems in the
	code, including consequential and incidental damages.

*/

#define     ALLOW_LEVEL_1_IO
#include    <fcntl.h>
#include    <termio.h>
#include    <sys/types.h>
#include    <sys/times.h>
#include    <errno.h>
#include    <stdio.h>
#include    <malloc.h>
#include    <sys/param.h>

#define     BEL     '\7'
#define     ESC     '\33'
#define     TIMEOUT  (-1)	      /* return value for timeout */
#define	    GOOD	0
#define	    BAD		(-1)
#define     TBSIZE 64
#define     COM_BUF_LENGTH 256	      /* size to allocate for com buffer */
 
static char tabbuf[NOFILE][TBSIZE];
static int tabi[NOFILE];

/*  I/O port definition structure - saved in configuration file */
struct iopb {
    int io_type;		      /* I/O type (serial/parallel) */
    struct termio new_params;	      /* new parameters */
    char devname[100];		      /* device name */ 
    short data_mask;		      /* data bit mask */
    short io_enable;		      /* set if port enabled */
    int devid;			      /* device id */
    struct termio save_params;        /* save existing terminal parameters */
    short comcount;		      /* character count in buffer */
    short commax;		      /* size of com buffer */
    char *combuf;		      /* pointer to com buffer */
    char *comptr;		      /* character pointer in com buffer */
    short parity		      /* requesed parity for this port */
};



struct iopb port = {0};	              /* Port definition blocks. */
short otimeron = 1;		      /* output timer on/off flag set on */
int stat;

/*  Initialize I/O for device idvc.  */
int sioinit()
{
	struct termio set_params;
	char *malloc();

	port.io_enable = 1;		/* port enabled */
	port.combuf = NULL;		/* no buffer allocated */
	port.comcount = 0;		/* nothing read yet */
	port.commax = COM_BUF_LENGTH;	/* max com buffer length */

        stat = GOOD; 		        /* start with good status */

   /* Open device, set new params, return -1 on TIMEOUT */
   /* Note, the following for loop around the open!  We cannot explain
      this, but if the command prompt area is configured off in the
      display driver, this open always gets hit with some interrupt.
      So we add a bit of protection.  Crazy Unix! 
    */

   for ( ;; ) {
    	if ((port.devid = open(port.devname, O_RDWR|O_NDELAY, 0)) < 0){
            if (errno == EINTR)         /* if interrupted */
            	continue;               /* simply try again! */
            else
            	stat = BAD;             /* something really went wrong */
      	}        
      	break;                          /* exit for loop */
   }

   if (stat == GOOD)                    /* if open was successful */
       	if(ioctl(port.devid, TCGETA, &port.save_params) < 0){ 
	   stat = BAD;			/* ioctl failed */
  	}

    if (stat == GOOD) {
    	set_params = port.save_params;	/* set old parameters */

    /* update with required changes */
    	set_params.c_iflag = port.new_params.c_iflag;
    	set_params.c_cflag = port.new_params.c_cflag;
    	set_params.c_oflag = port.new_params.c_oflag;
    	set_params.c_lflag = port.new_params.c_lflag;
    	set_params.c_cc[4] = port.new_params.c_cc[4];
    	set_params.c_cc[5] = port.new_params.c_cc[5]; 
    }

    if( ioctl(port.devid, TCSETAF, &port.new_params) < 0){
	stat = BAD;			/* ioctl failed */
    }

    if (stat == BAD) {                /* open or ioctl failed? */
        printf("Error opening %s. Device disabled.\n", port.devid);
        port.io_type = 0;             /* disable i/o on port */
        return 0;
    }

    port.combuf = malloc(COM_BUF_LENGTH); /* allocate buffer */

    if(port.combuf == NULL)
	return 0;

    return port.devid;
}

/*  Receive character, return -1 on TIMEOUT. */
int ior(timeout)
    int timeout;
{
    int count;
    int rtnchr;
    struct tms time_buf;
    long start_time = 0;
    int nreads = 0;

    if (port.io_type != 1) {
      	return TIMEOUT;
   }

    txflush(port.devid); /* HACK */

    while (port.comcount <= 0) {
      	errno = 0;		      /* clear error flag */
      	port.comcount = read(port.devid, port.combuf, port.commax); 
      	nreads++;
      	if (port.comcount <= 0) {
	    if (errno == EINTR) {     /* some interrupt try again */
	    continue;
	}
	if (timeout <= 0) {	      /* if no time to wait */
	    return TIMEOUT;	      /* bail out now */
	}
	if (start_time == 0) {       /* start timer */
	    start_time = times(&time_buf);  /* get current time */
	    timeout = (timeout * 60) / 1000;  /* convert to 1/60ths */
	} else if (timeout <= times(&time_buf) - start_time) { /* count down */
	    return TIMEOUT;
	}
     }
     port.comptr = port.combuf;
   }
   rtnchr = *port.comptr & port.data_mask;  /* get character and mask it */
   port.comptr++;		      /* increment character pointer */
   port.comcount--;		      /* decrement buffer count */
   return rtnchr;
}

/*  Write character.  */
int iow(data)
   char data;
{
   int status, timer = 0;

   if (port.io_type != 1)
      	return TIMEOUT;

   for ( ;; ) {
      	errno = 0;		      /* clear error status */
      	if ((status=write(port.devid, &data, 1)) == 1)  
	    break;
      	if (errno == EINTR) {	      /* write interrupted? */
	   continue;		      /* yes, try again */
      	}

      	/* Some error occurred, wait then try again */
      	if (timer++ < 20) {	      /* waited long enough? */
      	   sleep(1);		      /* wait a bit */
	   continue;		      /* no, try again */
      	}
      	if (errno != 0) 
      	printf("Cannot write to %s. Device disabled\n", port.devid);
      	port.io_type = 0;	      /* disable port */
      	return TIMEOUT;		      /* return error status */
   }
   return 0;
}

/*  Write a string of characters to the port */
int iown(data, nchars)
    char *data;			      /* beginning of string */
    int nchars;			      /* number of characters to send */
{
     int status, timer = 0;
     register int i;

     if (port.io_type != 1)
      	return TIMEOUT;

     /* Send one character at a time so that Unix does XON/XOFF correctly */
     for (i=0; i<nchars; ) {
      	errno = 0;		      /* clear error status */
      	if ((status=write(port.devid, data, 1)) == 1) {
	   i++;			      /* bump character output count */
	   data++;		      /* bump character pointer */
	   continue;
      	}
      	if (errno == EINTR) {	      /* write interrupted? */
	   continue;		      /* yes, try again */
      	}

      	/* Some error occurred, wait then try again */
      	if (timer++ < 20) {	      /* waited long enough? */
	   sleep(1);		      /* wait a bit */
	   continue;		      /* no, try again */
      	}

      	if (errno != 0)
      	printf("Cannot write to %s. Device disabled\n", port.devid);
      	port.io_type = 0;	      /* disable port */
      	return TIMEOUT;		      /* return error status */
    }
    return 0;
}

/*  Configure a port.  */
int cfgport(baudrate, parity, databits, stopbits)
  int baudrate, parity, databits, stopbits;
{

    static short data_bits[] = {0, 1, 3, 7, 0xF, 0x1F, 0x3F, 0x7F, 0xFF};

    static struct {
        unsigned short       speed;
        unsigned short  baud_value;
    }   baud_table[] = {
          {B50,50}, {B75,75}, {B110,110}, {B134,134}, {B150,150},
          {B200,200}, {B300,300}, {B600,600}, {B1200,1200}, {B1800,1800},
          {B2400,2400}, {B4800,4800}, {B9600,9600}, {EXTA,19200}
    };
#define num_bauds      (sizeof(baud_table)/sizeof(*baud_table))

    int devid, i;

    port.io_type = 0;			/* port undefined */
    port.devid = -1;			/* no active attached */

    strcpy(port.devname, "/dev/tty1a");

    for( ;; ){
        if ((devid = open(port.devname, O_RDWR, 0)) < 0){
                if (errno == EINTR)     /* if interrupted */
			continue;       /* simply try again! */
		else
	       	   stat = BAD;		/* something went wrong */
	}
	break;
    }

    if (ioctl(devid, TCGETA, &port.save_params) < 0)
	stat = BAD;     		/* ioctl failed */

    if (stat == BAD){
	if (devid >= 0) 
           close(devid);                /* release device */
	*(port.devname) = 0;		/* zap bad device name */
	return 0;
    }

    if (stat == GOOD)                   /* if open was successful */
	close(devid);

    /* set up the correct baud rate */
    for(i=num_bauds-1; i>=0; i--)
	if(baud_table[i].baud_value == baudrate)
	    break;
    if(i < 0){
	printf("Bad baudrate %d",baudrate);
	return 0;
    }

    /* set input/output speed */
    port.new_params.c_cflag = baud_table[i].speed;
    port.new_params.c_iflag = IGNBRK|IGNPAR; 
    port.new_params.c_cflag |= CREAD|CLOCAL;
    port.new_params.c_oflag = 0;
    port.new_params.c_lflag = NOFLSH;
    port.new_params.c_cc[4] = 1;
    port.new_params.c_cc[5] = 1;  

    if(databits == 5)
	port.new_params.c_cflag |= CS5;
    if(databits == 6)
	port.new_params.c_cflag |= CS6;
    if(databits == 7)
	port.new_params.c_cflag |= CS7;
    if(databits == 8){
	port.new_params.c_cflag |= CS8;
        port.parity = 0;                /* no parity generation */
    }

    if(stopbits == 2)
	port.new_params.c_cflag |= CSTOPB;

    if(parity == 2)
	port.new_params.c_cflag |= PARENB;
    if(parity == 1)
	port.new_params.c_cflag |= PARENB|PARODD;

    port.parity = parity;               /* save for iow routines */

    port.data_mask = data_bits[databits];
    port.io_type = 1;                   /* mark port active */

    return 1;
}

txflush(fd)
int fd;
{
        register int *tip;
        register int i;

        i = NOFILE;
        tip = &tabi[NOFILE];
        while (i--)
                if (*--tip) {
                        write(i, &tabbuf[i][0], *tip);
                        *tip = 0;
                }
}
